@@ -27,7 +27,7 @@ fn main() {
     });
 
     let mut wv = fltk_webview::Webview::create(false, &mut wv_win);
-    wv.navigate("http://wikipedia.com");
+    wv.navigate("https://google.com");
 
     // the webview handles the main loop
     wv.run();
@@ -46,6 +46,10 @@ use std::{
 };
 use webview_official_sys as wv;
 
+#[cfg(target_os = "macos")]
+#[macro_use]
+extern crate objc;
+
 pub(crate) trait FlString {
     fn safe_new(s: &str) -> CString;
 }
@@ -108,17 +112,67 @@ impl Webview {
             }
             #[cfg(target_os = "macos")]
             {
+                use objc::declare::MethodImplementation;
+                use objc::runtime::*;
+                use objc::{Encode, EncodeArguments};
+                extern "C" {
+                    pub fn objc_getMetaClass(name: *const std::os::raw::c_char) -> *mut Class;
+                }
+                extern fn my_method(_this: &Object, _cmd: Sel) -> bool {
+                    true
+                }
+                fn method_type_encoding(
+                    ret: &objc::Encoding,
+                    args: &[objc::Encoding],
+                ) -> std::ffi::CString {
+                    let mut types = format!(
+                        "{}{}{}",
+                        ret.as_str(),
+                        <*mut Object>::encode().as_str(),
+                        Sel::encode().as_str()
+                    );
+                    for enc in args {
+                        use std::fmt::Write;
+                        write!(&mut types, "{}", enc.as_str()).unwrap();
+                    }
+                    std::ffi::CString::new(types).unwrap()
+                }
+                fn add_method<F>(cls: *mut Class, sel: Sel, func: F)
+                where
+                    F: MethodImplementation<Callee = Object>,
+                {
+                    let encs = F::Args::encodings();
+                    let types = method_type_encoding(&F::Ret::encode(), encs.as_ref());
+                    unsafe {
+                        class_addMethod(
+                            cls,
+                            sel,
+                            func.imp(),
+                            types.as_ptr(),
+                        );
+                    }
+                }
                 let handle = win.raw_handle();
+                let cls2 = objc_getMetaClass("WKWebView\0".as_ptr() as _);
+                add_method(
+                    cls2,
+                    sel!(did_view_resolution_change),
+                    my_method as extern fn(&Object, Sel) -> bool,
+                );
                 inner = wv::webview_create(debug as i32, handle as *mut raw::c_void);
+                let cls = object_getClass(wv::webview_get_window(inner) as _) as *mut Class;
+                add_method(
+                    cls,
+                    sel!(did_view_resolution_change),
+                    my_method as extern fn(&Object, Sel) -> bool,
+                );
             }
             #[cfg(target_os = "linux")]
             {
                 pub enum GdkWindow {}
-                pub enum GtkWidget {}
                 pub enum GtkWindow {}
                 pub enum Display {}
                 extern "C" {
-                    pub fn gtk_init(argc: *mut raw::c_int, argv: *mut *mut raw::c_char);
                     pub fn my_get_win(wid: *mut GtkWindow) -> *mut GdkWindow;
                     pub fn my_get_xid(w: *mut GdkWindow) -> u64;
                     pub fn XReparentWindow(
@@ -129,19 +183,18 @@ impl Webview {
                         y: i32,
                     );
                     pub fn XMapWindow(display: *mut Display, w: u64);
-                    pub fn gtk_widget_set_size_request(wid: *mut GtkWidget, w: i32, h: i32);
+                    pub fn XFlush(disp: *mut Display);
                 }
-                gtk_init(&mut 0, std::ptr::null_mut());
                 inner = wv::webview_create(debug as i32, std::ptr::null_mut() as _);
                 assert!(!inner.is_null());
-                wv::webview_set_size(inner, win.w(), win.h(), 0);
                 let temp_win = wv::webview_get_window(inner);
                 let temp = my_get_win(temp_win as _);
                 assert!(!temp.is_null());
                 let xid = my_get_xid(temp as _);
-                gtk_widget_set_size_request(temp_win as _, win.x(), win.h());
-                XReparentWindow(app::display() as _, xid, win.raw_handle(), win.x(), win.y());
+                XReparentWindow(app::display() as _, xid, win.top_window().unwrap().raw_handle(), win.x(), win.y());
                 XMapWindow(app::display() as _, xid);
+                XFlush(app::display() as _);
+                wv::webview_set_size(inner, win.w(), win.h(), 0);
             }
         }
         assert!(!inner.is_null());
@@ -235,7 +288,9 @@ impl Webview {
 
     /// Run the main loop of the webview
     pub fn run(&self) {
-        unsafe { wv::webview_run(*self.inner) }
+        unsafe {
+            wv::webview_run(*self.inner) 
+        }
     }
 
     /// Set the size of the webview window
